"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var nodePath = require("path");
var utils_1 = require("./utils");
exports.default = (function (t, type, name, rootPath, params, bodyPath, async) {
    var context = null;
    bodyPath.traverse({
        Identifier: function (p) {
            // Does it use `state` from `import {state} from "./model";`
            if (p.isReferencedIdentifier()) {
                var binding = p.scope.getBinding(p.node.name);
                if (binding != null) {
                    var importPath = binding.path;
                    if (t.isImportSpecifier(importPath.node) &&
                        utils_1.isInUniverse(importPath.node.imported.name)) {
                        var importDeclarationPath = importPath.parentPath;
                        if (t.isImportDeclaration(importDeclarationPath.node)) {
                            var source = importDeclarationPath.node.source;
                            if (source.value.startsWith(".") &&
                                /^model(\.(j|t)s)?$/.test(nodePath.basename(source.value))) {
                                var importSpecifiers = importDeclarationPath.node.specifiers.filter(function (specifier) { return t.isImportSpecifier(specifier); });
                                context = {
                                    importType: "specifiers",
                                    importDeclarationPath: importDeclarationPath,
                                    universe: t.objectPattern(importSpecifiers
                                        .filter(function (specifier) {
                                        return utils_1.isInUniverse(specifier.imported.name);
                                    })
                                        .map(function (specifier) {
                                        return t.objectProperty(specifier.imported, specifier.local, false, specifier.imported.name === specifier.local.name);
                                    })),
                                };
                                p.stop();
                            }
                        }
                    }
                }
            }
        },
        MemberExpression: function (p) {
            // Does it use `prodo.state` from `import * as prodo from "./model";`
            if (!p.node.computed && utils_1.isInUniverse(p.node.property.name)) {
                var objectPath = p.get("object");
                if (t.isIdentifier(objectPath.node) &&
                    objectPath.isReferencedIdentifier()) {
                    var binding = objectPath.scope.getBinding(objectPath.node.name);
                    if (binding != null) {
                        var importPath = binding.path;
                        if (t.isImportNamespaceSpecifier(importPath.node)) {
                            var importDeclarationPath = importPath.parentPath;
                            if (t.isImportDeclaration(importDeclarationPath.node)) {
                                var source = importDeclarationPath.node.source;
                                if (source.value.startsWith(".") &&
                                    /^model(\.(j|t)s)?$/.test(nodePath.basename(source.value))) {
                                    context = {
                                        importType: "namespace",
                                        importDeclarationPath: importDeclarationPath,
                                        universe: t.identifier(importPath.node.local.name),
                                    };
                                    p.stop();
                                }
                            }
                        }
                    }
                }
            }
        },
    });
    if (context == null) {
        // Doesn't use the universe, so don't change anything.
        return;
    }
    // https://stackoverflow.com/questions/44147937/property-does-not-exist-on-type-never
    context = context;
    rootPath.replaceWith(t.variableDeclaration("const", [
        t.variableDeclarator(t.identifier(name), t.callExpression(t.memberExpression(context.importType === "namespace"
            ? t.memberExpression(context.universe, t.identifier("model"))
            : t.identifier("model"), t.identifier(type === "action" ? "action" : "connect")), [
            t.arrowFunctionExpression([context.universe], t.arrowFunctionExpression(params, bodyPath.node, async)),
            t.stringLiteral(name),
        ])),
    ]));
    if (context.importType === "specifiers" &&
        !context.importDeclarationPath.node.specifiers.some(function (specifier) {
            return t.isImportSpecifier(specifier) &&
                specifier.imported.name === "model" &&
                specifier.local.name === "model";
        })) {
        context.importDeclarationPath.unshiftContainer("specifiers", t.importSpecifier(t.identifier("model"), t.identifier("model")));
    }
});
//# sourceMappingURL=inject-universe.js.map